# author: tori31001 at gmail.com
# website: http://blenderfbx.render.jp/

bl_info = {
    "name": "Another Autodesk FBX Import/Export",
    "author": "Kazuma Hatta",
    "version": (0, 8, 1),
    "blender": (2, 6, 5),
    "location": "File > Import/Export > Another Autodesk FBX",
    "description": "Import-Export FBX, io: mesh, skeleton",
    "warning": "",
    "wiki_url": "",
    "tracker_url": "",
    "category": "Import-Export"}
    
if "bpy" in locals():
    import imp
    if "import_another_fbx" in locals():
        imp.reload(import_another_fbx)
    if "export_another_fbx" in locals():
        imp.reload(export_another_fbx)

import bpy

from bpy.props import (StringProperty,
                       BoolProperty,
                       FloatProperty,
                       EnumProperty,
                       )

from bpy_extras.io_utils import (ImportHelper,
                                 ExportHelper,
                                 path_reference_mode
                                 )

class FbxExportOperator(bpy.types.Operator, ExportHelper):
    bl_idname = "export_scene.anothor_fbx"
    bl_label = "Anothor Fbx Exporter(.fbx)"
    
    filename_ext = ".fbx"
    fliter_glob = bpy.props.StringProperty(default="*.fbx", options={'HIDDEN'})

    filepath = bpy.props.StringProperty(subtype="FILE_PATH")
        
    path_mode = path_reference_mode

    is_text = BoolProperty(\
        name="text",\
        description="export text fbx",\
        default=False,\
        )

    only_selected = BoolProperty(\
        name="only selected",\
        description="export only selected objects",\
        default=False,\
        )

    imported_node_property = BoolProperty(\
        name="export imported node properties",\
        description="export imported node props including local trans/rot/scale and pivot.",\
        default=False,\
        )

    fit_node_length = BoolProperty(\
        name="fit node length",\
        description="Even if export node properties, apply bone length",\
        default=False,\
        )

    def execute(self, context):
        import os, sys
        cmd_folder = os.path.dirname(os.path.abspath(__file__))
        if cmd_folder not in sys.path:
            sys.path.insert(0, cmd_folder)
        import export_another_fbx
        export_another_fbx.export_fbx(self.filepath, bpy.context, \
                self.is_text,\
                self.only_selected,\
                self.imported_node_property,\
                self.fit_node_length)

        return {'FINISHED'}

    def invoke(self, context, event):
        context.window_manager.fileselect_add(self)
        return {'RUNNING_MODAL'}

    def draw(self, context):
        layout = self.layout
        row = layout.row(align=True)
        row.prop(self, "is_text")
        row = layout.row(align=True)
        row.prop(self, "only_selected")
        row = layout.row(align=True)
        row.prop(self, "imported_node_property")
        row = layout.row(align=True)
        row.prop(self, "fit_node_length")
  
class FbxImportOperator(bpy.types.Operator, ImportHelper):
    bl_idname = "import_scene.anothor_fbx"
    bl_label = "Anothor Fbx Importer(.fbx)"
    
    filename_ext = ".fbx"
    fliter_glob = bpy.props.StringProperty(default="*.fbx", options={'HIDDEN'})

    filepath = bpy.props.StringProperty(subtype="FILE_PATH")
        
    path_mode = path_reference_mode

    def execute(self, context):
        import os, sys
        cmd_folder = os.path.dirname(os.path.abspath(__file__))
        if cmd_folder not in sys.path:
            sys.path.insert(0, cmd_folder)
        import import_another_fbx
        import_another_fbx.import_fbx(self.filepath, bpy.context)
        return {'FINISHED'}

    def invoke(self, context, event):
        context.window_manager.fileselect_add(self)
        return {'RUNNING_MODAL'}

#
# Registration
#
def menu_func_import(self, context):
    self.layout.operator(FbxImportOperator.bl_idname, text="Another Autodesk FBX (.fbx)")

def menu_func_export(self, context):
    self.layout.operator(FbxExportOperator.bl_idname, text="Another Autodesk FBX (.fbx)")
    
def register():
    bpy.utils.register_module(__name__)
    bpy.types.INFO_MT_file_import.append(menu_func_import)
    bpy.types.INFO_MT_file_export.append(menu_func_export)

def unregister():
    bpy.utils.unregister_module(__name__)
    bpy.types.INFO_MT_file_import.remove(menu_func_import)
    bpy.types.INFO_MT_file_export.remove(menu_func_export)
    
if __name__ == "__main__":
    register()