# author: tori31001 at gmail.com
# website: http://blenderfbx.render.jp/
import bpy
import os
import sys
import math
import mathutils
import platform
from bpy_extras.io_utils import unpack_list

if "64bit" in platform.architecture()[0]:
    bl_folder = os.path.dirname(os.path.abspath(__file__))
    bl_folder = os.path.join(bl_folder, "win64bit")
    if sys.version_info[:2] == (3, 3):
        bl_folder = os.path.join(bl_folder, "python33")
    if sys.version_info[:2] == (3, 2):
        bl_folder = os.path.join(bl_folder, "python32")
    if bl_folder not in sys.path:
        sys.path.insert(0, bl_folder)
    import BL
else:
    bl_folder = os.path.dirname(os.path.abspath(__file__))
    bl_folder = os.path.join(bl_folder, "win32bit")
    if sys.version_info[:2] == (3, 3):
        bl_folder = os.path.join(bl_folder, "python33")
    if sys.version_info[:2] == (3, 2):
        bl_folder = os.path.join(bl_folder, "python32")
    if bl_folder not in sys.path:
        sys.path.insert(0, bl_folder)
    import BL

def to_blender_matrix(bl_mat):
    mat = mathutils.Matrix()
    for i, col in enumerate(bl_mat):
        for k, val in enumerate(col):
            mat[k][i] = val
    return mat

def unpack_face_list_fix(list_of_tuples):
    #allocate the entire list
    flat_ls = [0] * (len(list_of_tuples) * 4)
    i = 0

    for t in list_of_tuples:
        if len(t) == 3:
            t = t[0], t[1], t[2], 0
        else:
            t = t[0], t[1], t[2], t[3]
        flat_ls[i:i + len(t)] = t
        i += 4
    return flat_ls

def import_mesh(file, obj, armature_obj, context):
    SWAP_YZ_MATRIX = mathutils.Matrix.Rotation(math.radians(90.0), 4, 'X')
    
    for mesh_container in obj.mesh_list():
        mesh = mesh_container.data()
        me = bpy.data.meshes.new(mesh.name())
        me_obj = bpy.data.objects.new(mesh.name(),me)
        me_obj.select = True
        context.scene.objects.link(me_obj)
        context.scene.objects.active = me_obj
        
        try:
            me.vertices.add(len(mesh.vertex_list()))
            me.vertices.foreach_set("co", unpack_list(mesh.vertex_list()))
            me.vertices.foreach_set("normal", unpack_list(mesh.normal_list()))
            me.tessfaces.add(len(mesh.vertex_index_list()))
            me.tessfaces.foreach_set("vertices_raw", unpack_face_list_fix(mesh.vertex_index_list()))
        except:
            pass

        # material
        for i, material in enumerate(mesh.material_list()):
            mat = bpy.data.materials.new(material.name())
            bpy.ops.object.material_slot_add()
            me_obj.material_slots[i].material = mat
            
            if len(mat.diffuse_color) == len(material.diffuse()):
                mat.diffuse_color = material.diffuse()
            if len(mat.specular_color) == len(material.specular()):
                mat.specular_color = material.specular()
            if len(mat.mirror_color) == len(material.emissive()):
                mat.mirror_color = material.emissive()
            mat.alpha = material.transparency_factor()
            mat.specular_intensity = material.shininess()
            
            # texture
            for k, texture in enumerate(material.texture_list()):                           
                tex = mat.texture_slots.create(k)
                tex.texture = bpy.data.textures.new(texture.name(), type='IMAGE')
                tex.texture_coords = 'UV'
                tex.use = True
                tex.use_map_color_diffuse = True
                fbx_base_path, fbx_file_name = os.path.split(file)
                fbx_texture_file_path = texture.file_name()
                fbx_garbage_path, fbx_texture_name = os.path.split(fbx_texture_file_path)
                base_norm = os.path.normpath(fbx_base_path)
                garbage_norm = os.path.normpath(fbx_garbage_path)
                if base_norm in garbage_norm and os.path.exists(fbx_texture_file_path):
                    texture_file_path = fbx_texture_file_path
                else:
                    texture_file_path = bpy.path.abspath(os.path.join(fbx_base_path, fbx_texture_name))
                texture_file_path = os.path.normpath(texture_file_path)
                if os.path.exists(texture_file_path):
                    tex.texture.image = bpy.data.images.load(texture_file_path)

        # material index
        for i, index in enumerate(mesh.material_index_list()):
            me.tessfaces[i].material_index = index
            
        # uv
        layered_uv_list = mesh.layered_uv_list()
        for i, uv_layer in enumerate(layered_uv_list):
            me.tessface_uv_textures.new()
            uvs = uv_layer
            if len(uvs) == 0:
                continue
            ttex = me.tessface_uv_textures[i]
            count = 0
            for k, face in enumerate(mesh.vertex_index_list()):
                tface = ttex.data[k]
                tessFace = me.tessfaces[k]
                mat = me_obj.material_slots[tessFace.material_index].material
                if mat.texture_slots[i] != None:
                    tface.image = mat.texture_slots[i].texture.image
                vcount = len(face)
                if vcount == 3:
                    tface.uv1 = uvs[count]
                    count = count + 1
                    tface.uv2 = uvs[count]
                    count = count + 1
                    tface.uv3 = uvs[count]
                    count = count + 1
                elif vcount == 4:
                    tface.uv1 = uvs[count]
                    count = count + 1
                    tface.uv2 = uvs[count]
                    count = count + 1
                    tface.uv3 = uvs[count]
                    count = count + 1
                    tface.uv4 = uvs[count]
                    count = count + 1
                else:
                    count = count + vcount
            
        me.update(calc_edges=True)
        global_trans = to_blender_matrix(mesh.global_transform())
        me_obj.matrix_basis = SWAP_YZ_MATRIX * global_trans
        
        # calculate normalized weight
        normalized_weights = {}
        for i, skin in enumerate(mesh.skin_list()):
            for k, cluster in enumerate(skin.cluster_list()):
                for n, index in enumerate(cluster.index_list()):
                    weight = cluster.weight_list()[n]
                    if not (index in normalized_weights):
                        normalized_weights[index] = 0.0
                    normalized_weights[index] = normalized_weights[index] + weight
                    
        # create bone group
        for i, skin in enumerate(mesh.skin_list()):                     
            if mesh.id() != skin.geometry_node_id():
                continue
            
            for k, cluster in enumerate(skin.cluster_list()):
                link_node = cluster.link_node()
                if link_node == None:
                    continue
                link_name = cluster.link_node().name().split(' ')[-1]

                # use existing group or new
                group = None
                if link_name in me_obj.vertex_groups.keys():
                    group = me_obj.vertex_groups[link_name]
                if group == None:
                    group = me_obj.vertex_groups.new(link_name)
                    
                for n, index in enumerate(cluster.index_list()):
                    weight = cluster.weight_list()[n]
                    
                    if normalized_weights[index] > 0:
                        group.add([index], weight/normalized_weights[index], 'ADD')
                    else:
                        group.add([index], weight, 'ADD')
        # modifier
        if armature_obj != None:
            modifier = me_obj.modifiers.new('Armature', 'ARMATURE')
            modifier.object = armature_obj
    
def import_armature(file, obj, context):
    SWAP_YZ_MATRIX = mathutils.Matrix.Rotation(math.radians(90.0), 4, 'X')
    
    if len(obj.skeleton_list()) == 0:
        return
    
    armature = bpy.data.armatures.new("armature")
    armature_obj = bpy.data.objects.new("armature", armature)
    armature_obj.select = True
    context.scene.objects.link(armature_obj)
    context.scene.objects.active = armature_obj
    armature.draw_type = 'STICK'
    armature.show_axes = True
    armature_obj.show_x_ray = True
    
    if not bpy.ops.object.mode_set.poll():
        return
    
    bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
    bpy.ops.object.mode_set(mode='EDIT', toggle=False)

    # create {id, bone} dic
    bone_dic = {}
    for skeleton_contanier in obj.skeleton_list():
        skeleton = skeleton_contanier.data()
        id = skeleton.id()
        parent = skeleton.parent()
        bone = armature.edit_bones.new(skeleton.name())
        bone_dic[id] = bone
        
        mat = to_blender_matrix(skeleton.global_transform())
        bone.transform(SWAP_YZ_MATRIX * mat)
            
    # set parent
    for skeleton_contanier in obj.skeleton_list():
        skeleton = skeleton_contanier.data()
        bone = bone_dic[skeleton.id()]
        if skeleton.parent_id() in bone_dic.keys():
            bone.parent = bone_dic[skeleton.parent_id()]

    bpy.types.EditBone.fbx_local_translation_x = bpy.props.FloatProperty()
    bpy.types.EditBone.fbx_local_translation_y = bpy.props.FloatProperty()
    bpy.types.EditBone.fbx_local_translation_z = bpy.props.FloatProperty()
    bpy.types.EditBone.fbx_local_rotation_x = bpy.props.FloatProperty()
    bpy.types.EditBone.fbx_local_rotation_y = bpy.props.FloatProperty()
    bpy.types.EditBone.fbx_local_rotation_z = bpy.props.FloatProperty()
    bpy.types.EditBone.fbx_local_scaling_x = bpy.props.FloatProperty()
    bpy.types.EditBone.fbx_local_scaling_y = bpy.props.FloatProperty()
    bpy.types.EditBone.fbx_local_scaling_z = bpy.props.FloatProperty()
    
    bpy.types.EditBone.fbx_rotation_offset = bpy.props.FloatVectorProperty()
    bpy.types.EditBone.fbx_rotation_pivot = bpy.props.FloatVectorProperty()
    bpy.types.EditBone.fbx_pre_rotation = bpy.props.FloatVectorProperty()
    bpy.types.EditBone.fbx_post_rotation = bpy.props.FloatVectorProperty()
    bpy.types.EditBone.fbx_scaling_offset = bpy.props.FloatVectorProperty()
    bpy.types.EditBone.fbx_scaling_pivot = bpy.props.FloatVectorProperty()
    bpy.types.EditBone.fbx_geometric_translation = bpy.props.FloatVectorProperty()
    bpy.types.EditBone.fbx_geometric_rotation = bpy.props.FloatVectorProperty()
    bpy.types.EditBone.fbx_geometric_scaling = bpy.props.FloatVectorProperty()
        
    # detect bone position and length
    for skeleton_contanier in obj.skeleton_list():
        skeleton = skeleton_contanier.data()
        bone = bone_dic[skeleton.id()]
        if len(bone.children) == 1:
            bone.tail = bone.children[0].head
        else:
            bone.tail = bone.head + mathutils.Vector([0,0,-0.1])

        bone.use_inherit_scale = False
        bone.fbx_local_translation_x = skeleton.local_translation()[0]
        bone.fbx_local_translation_y = skeleton.local_translation()[1]
        bone.fbx_local_translation_z = skeleton.local_translation()[2]
        bone.fbx_local_rotation_x = skeleton.local_rotation()[0]
        bone.fbx_local_rotation_y = skeleton.local_rotation()[1]
        bone.fbx_local_rotation_z = skeleton.local_rotation()[2]
        bone.fbx_local_scaling_x = skeleton.local_scaling()[0]
        bone.fbx_local_scaling_y = skeleton.local_scaling()[1]
        bone.fbx_local_scaling_z = skeleton.local_scaling()[2]
        bone.fbx_rotation_offset = skeleton.rotation_offset()[0:3]
        bone.fbx_rotation_pivot = skeleton.rotation_pivot()[0:3]
        bone.fbx_pre_rotation = skeleton.pre_rotation()[0:3]
        bone.fbx_post_rotation = skeleton.post_rotation()[0:3]
        bone.fbx_scaling_offset = skeleton.scaling_offset()[0:3]
        bone.fbx_scaling_pivot = skeleton.scaling_pivot()[0:3]
        bone.fbx_geometric_translation = skeleton.geometric_translation()[0:3]
        bone.fbx_geometric_rotation = skeleton.geometric_rotation()[0:3]
        bone.fbx_geometric_scaling = skeleton.geometric_scaling()[0:3]
        
    bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
        
    return armature_obj

def import_fbx(file, context):
    fbx = BL.BLFbx()
    setting = BL.BLFbxSetting()
    setting.set_bl_imp_bool_prop(BL.BLFbxSetting.BLImpTriangulate, True)
    setting.set_bl_imp_bool_prop(BL.BLFbxSetting.BLImpNurbs, True)
    setting.set_bl_imp_bool_prop(BL.BLFbxSetting.BLImpPatch, True)
    
    obj = fbx.load(file, setting)
    if obj == None:
        return
    
    armature_obj = import_armature(file, obj, context)
    import_mesh(file, obj, armature_obj, context)

    bpy.ops.object.shade_smooth()
     
#bpy.utils.register_class(FbxImportOperator)
#bpy.ops.import_scene.anothor_fbx('INVOKE_DEFAULT')